using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Game : MonoBehaviour {
	
	public GameObject[] pieces;
	
	public int[][] leveldat = 
	{
		new int[]{8,0},
		new int[]{3,0,8,0},
		new int[]{7,0,8,0,4,0,9,0},
		new int[]{7,1,8,2},
		new int[]{3,1,8,2,9,3},
		new int[]{10,4,9,5,6,5,8,4},
		new int[]{8,6,4,6,7,6,3,6},
	};
	
	public List<GameObject> curpieces;
	
	public int level=0;
	public float left=11;
	public float right=-11;
	
	public GameObject deadtext;
	
	//octagon is element 4
	// Use this for initialization
	void Awake () {
		foreach (GameObject go in pieces)
		{
			go.active=false;
		}
		curpieces = new List<GameObject>();
		UpdateScore();
		SpawnLevel();
	}
	
	void SpawnLevel()
	{
		for (int i=curpieces.Count-1;i>=0;i--)
		{
			DestroyImmediate(curpieces[i]);
		}
		curpieces = new List<GameObject>();
		
		if (level<leveldat.Length)
		{
			/*shuffle its entries*/
			int n = leveldat[level].Length/2;
	        while (n > 1) 
	        {
	            int k = Random.Range(0,n);
	            n--;
	            int ta = leveldat[level][n*2+0];
	            int tb = leveldat[level][n*2+1];
	            leveldat[level][n*2+0]=leveldat[level][k*2+0];
				leveldat[level][n*2+1]=leveldat[level][k*2+1];				
	            leveldat[level][k*2+0] = ta;
	            leveldat[level][k*2+1] = tb;
	        }
	
				
			
			for (int i=0;i<leveldat[level].Length/2;i++)
			{
				float pos;
				if (leveldat[level].Length/2==1)
				{
					pos=0;
				}
				else
				{
					pos = left + i*((right-left)/(leveldat[level].Length/2-1));
				}
				int pieceindex = leveldat[level][i*2+0]-3;
//				Debug.Log(pieceindex);
				GameObject go = (GameObject)
					Instantiate(pieces[pieceindex],new Vector3(0.0f,pos,0),Quaternion.identity);
				go.transform.parent=transform;
				go.transform.rotation *= Quaternion.AngleAxis(Random.Range(0.0f,360.0f),Vector3.forward);
				go.GetComponent<Rotate>().RPS = (float)leveldat[level][i*2+1]/10.0f;
				curpieces.Add(go);
			}
		}
		else
		{
			int piececount = Random.Range(3,Mathf.Min(5+(level-leveldat.Length)/3,7));
			int eightindex = Random.Range(0,piececount);
			for (int i=0;i<piececount;i++)
			{
				int piecenum = Random.Range(0,pieces.Length-1);
				if (piecenum>=5) piecenum++;
				if (eightindex==i)
					piecenum=5;
				int rotvelocity = Random.Range(2,7+(level-leveldat.Length));
				float pos = left + i*((right-left)/(piececount-1));
				
				GameObject go = (GameObject)
					Instantiate(pieces[piecenum],new Vector3(0.0f,pos,0),Quaternion.identity);
				go.transform.parent=transform;
				go.transform.rotation *= Quaternion.AngleAxis(Random.Range(0.0f,360.0f),Vector3.forward);
				go.GetComponent<Rotate>().RPS = rotvelocity/10.0f;
				curpieces.Add(go);
				
			}
		}
	}
	
	void GameOver()
	{
		
		for (int i=curpieces.Count-1;i>=0;i--)
		{
			DestroyImmediate(curpieces[i]);
		}
		curpieces = new List<GameObject>();
		if (PlayerPrefs.GetInt("highscore")<level)
		{
			PlayerPrefs.SetInt("highscore",level);
		}
		UpdateScore();
		deadtext.active=true;
		dead=true;
	}
	
	void NextLevel()
	{
		deadtext.active=false;				
		level++;
		UpdateScore();
		SpawnLevel();
	}
	
	
	public void UpdateScore()
	{
		int highscore = 10 * PlayerPrefs.GetInt("highscore",0);
		highscoretxt.text = "high : "+(highscore);
		scoretxt.text = (10*level).ToString();
	}
	
	public TextMesh highscoretxt;
	public TextMesh scoretxt;
	private bool dead = false;
	
	// Update is called once per frame
	void Update () {
		if (Input.GetKeyDown(KeyCode.Escape))
		{
			Application.Quit();
		}
		
		if (dead)
		{
			if (Input.GetMouseButtonDown(0))
			{
				dead=false;
				deadtext.active=false;
				level=0;
				UpdateScore();
				SpawnLevel();
			}
			return;
		}
		
		if (Input.GetMouseButtonDown(0))
		{
			Ray r = Camera.main.ScreenPointToRay(Input.mousePosition);
			RaycastHit hit;
			if (Physics.Raycast(r,out hit))
			{
				if (hit.collider.GetComponent<Octagon>()==null)
				{
					GameOver();
				}
				else
				{
					NextLevel();
				}
			}
		}
	}
}
